#!/bin/ksh -p
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2009
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# VERSION: db2_v97fp4 s110330
# NAME: db2V97_monitor.ksh db2instance [NN] [verbose]
#
# (%W%) %E%  %U%
#
# FUNCTION: Monitor method for DB2 in TSA HA cluster
#
# Return 1 if specified DB2 instance is online
# Return 2 otherwise
#
#-----------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

export CT_MANAGEMENT_SCOPE=2

DB2INSTANCE=${1}
NN=${2:-0}
VERBOSE=${3:-noverbose}

if [[ "$VERBOSE" == "verbose" ]]; then
   :
   exec  2> /dev/null
else
   #  Close stdout   and   stderr
   exec  2> /dev/null
   exec  1> /dev/null
   set   +x
fi

if [[ -z $DB2INSTANCE ]]; then
   logger -i -p err -t $0 "Error: Must specify instance name"
   echo 2
   return 2
fi

if [[ -z ${INSTHOME} ]] ; then
   userhome=~${DB2INSTANCE?}
   eval userhome=$userhome
   INSTHOME=${userhome}
fi
if [[ -z ${INSTHOME} ]] ; then
   INSTHOME=$(cat /etc/passwd | grep "^${DB2INSTANCE?}:" | cut -f 6 -d \:)
fi
if [[ -z ${INSTHOME} ]] ; then
   logger -i -p err -t $0 "There is no home directory defined for ${DB2INSTANCE?}"
   echo 2
   return 2
fi

INSTHOME=${INSTHOME#*=}
INSTHOME=${INSTHOME%%*( )}

###################################################################
# Start main...

# Comment following to not use ps
bypassHomeDir=TRUE
if [[ -z $bypassHomeDir ]]; then
   cd ${INSTHOME?}/sqllib
   cd /
   if [[ -x $INSTHOME/sqllib/bin/db2gcf ]]; then
      su - ${DB2INSTANCE} -c "$INSTHOME/sqllib/bin/db2gcf -s -p ${NN?} -i ${DB2INSTANCE?} > /dev/null 2> /dev/null"
      rc=$?
   else
      p_pid=$(su - ${DB2INSTANCE?} -c "db2nps ${NN?}" | grep -c db2sys)
      if [[ $p_pid == 0 ]]; then
         rc=1
      else
         rc=0
      fi
   fi
else
   # If home dir not accessible, use plain old ps ...
   p_out=$(ps -u ${DB2INSTANCE?} -o args | egrep "^db2sysc ${NN?}[ ]*$|^db2sysc[ ]*$|^\[db2sysc\]")

   p_pid=$(echo $p_out | grep -c "db2sysc ${NN?}")
   if [[ $p_pid == 0 && $NN -eq 0 ]]; then
      p_pid=$(echo $p_out | grep -c "db2sysc")
   fi

   if [[ $p_pid == 0 ]]; then
      p_pid=$(echo $p_out | grep -c "\[db2sysc\]")
      if [[ $p_pid != 0 ]]; then
         logger -i -p err -t $0 "ps returns [db2sysc]: returning 0"
         echo 0
         return 0
      fi
      rc=1
   else
      rc=0
   fi
fi

# Logic for rc is 0 if db2 is ok, non-zero otherwise
# Change logic to conform to cluster's expectations...
if [[ $rc == 0 ]]; then
   cluster_rc=1
else
   cluster_rc=2
fi

# Allow DPF cleanup if required
if [[ $cluster_rc == 2 ]]; then
  p_pid=$(ps -e -o args | grep -c "^db2wdog [0-9]\+ [0-9]\+")
  if [[ $p_pid > 0 ]]; then
     DB2_CLEANUP_TIME=30
     logger -i -p notice -t $0 "DB2 cleaning up: sleep $DB2_CLEANUP_TIME seconds ($DB2INSTANCE, $NN)"
     sleep $DB2_CLEANUP_TIME
  fi
fi

logger -i -p debug -t $0 "Returning $cluster_rc ($DB2INSTANCE, $NN)"
echo "rc=$cluster_rc"
return $cluster_rc
