#!/bin/ksh -p
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2009
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# VERSION: db2_v97fp4 s110330
# NAME: db2V97_stop.ksh db2instance [verbose]
#
# (%W%) %E%  %U%
#
# FUNCTION: Stop method for DB2 in TSA HA cluster
#
#-----------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

export CT_MANAGEMENT_SCOPE=2

DB2INSTANCE=${1}
NN=${2:-0}
VERBOSE=${3:-noverbose}

PROGNAME=$(basename $0)
PROGPATH=$(dirname $PROGNAME)

isHadrPrimary=0

logger -i -p notice -t $PROGNAME "Entered $0, $DB2INSTANCE, $NN"

if [[ "$VERBOSE" == "verbose" ]]; then
   typeset -ft $(typeset +f)
   set -x
else
   # Close stdout and stderr
   exec 2> /dev/null
   exec 1> /dev/null
   set +x
fi


###################################################################
# Start main error checking...
if [[ -z $DB2INSTANCE ]]; then
   logger -i -p err -t $PROGNAME "Error: Must specify instance name"
   return 0
fi

if [[ -z ${INSTHOME} ]] ; then
   userhome=~${DB2INSTANCE?}
   eval userhome=$userhome
   INSTHOME=${userhome}
fi
if [[ -z ${INSTHOME} ]] ; then
   INSTHOME=$(cat /etc/passwd | grep "^${DB2INSTANCE?}:" | cut -f 6 -d \:)
fi
if [[ -z ${INSTHOME} ]] ; then
   logger -i -p err -t $PROGNAME "Error: There is no home directory defined for ${DB2INSTANCE?}"
   return 0
fi

INSTHOME=${INSTHOME#*=}
INSTHOME=${INSTHOME%%*( )}
###################################################################
# main ...

# Ensure home directory is accessible ...
/bin/ksh -c "cd $INSTHOME/sqllib/bin; touch $INSTHOME/sqllib/tmp/.tmp.$NN; rm -f $INSTHOME/sqllib/tmp/.tmp.$NN" &
ProcNum=$!
sleep 2
kill -0 ${ProcNum} 2> /dev/null
ret=$?
kill -9 ${ProcNum} 2> /dev/null

if [[ $ret == 0 ]]; then
   # logger -i -p err -t $PROGNAME "$INSTHOME may not be accessible, Will do a hard kill of ( $DB2INSTANCE, $NN )"
   kill -9 $(ps -o pid,comm -u $DB2INSTANCE | grep "db2[a-z]" | awk '{print $1}') 2> /dev/null > /dev/null
   rc=3
   logger -i -p info -t $PROGNAME "Returning $rc from $0 ( $DB2INSTANCE, $NN )"
   return $rc
fi

# Do not need to stop if no process model
p_pid=$(ps -u ${DB2INSTANCE?} -o args | grep -c "^db2sysc ${NN?}[ ]*$")
if [[ $p_pid == 0 && $NN -eq 0 ]]; then
   p_pid=$(ps -u ${DB2INSTANCE?} -o args | grep -v "^db2sysc [0-9]" | grep -c "^db2sysc")
fi

if [[ $p_pid == 0 ]]; then
   logger -i -p notice -t $PROGNAME "Returning $rc from $0 ( $DB2INSTANCE, $NN ).  Partition already stopped."
   return 0
fi

# If single partition, check for HADR db's
DB2NODES_CFG=${INSTHOME?}/sqllib/db2nodes.cfg
if test -r $DB2NODES_CFG ; then
   nln=$(grep -v "^[ ]*$" $DB2NODES_CFG | wc -l | awk '{print $1}')
else
   nln=1
fi

if [ $nln -eq 1 ]; then
   dbl1=/tmp/dblist1.${DB2INSTANCE?}.${NN?}.out
   rm -f ${dbl1?} 2> /dev/null
   touch ${dbl1?} 2> /dev/null
   su - ${DB2INSTANCE?} -c "db2 list database directory" | grep "Database name" \
   |  awk '{print $4}' > ${dbl1?}

   while read db; do
      rc=$(su - ${DB2INSTANCE} -c "db2pd -hadr -db ${db?}" | grep -c ^'Primary ')
      if [[ $rc -eq 1 ]]; then
         isHadrPrimary=1
      fi
   done < ${dbl1?}
   rm -f ${dbl1?}
   if [ $isHadrPrimary -eq 1 ]; then  
      # Use db2gcf -k and for good measure, set rc such that kill is invoked later in this script
      su - ${DB2INSTANCE} -c "$INSTHOME/sqllib/bin/db2gcf -t 100 -k -p ${NN?} -i ${DB2INSTANCE?} -L"
      rc=1
   else
      su - ${DB2INSTANCE} -c "$INSTHOME/sqllib/bin/db2gcf -t 100 -d -p ${NN?} -i ${DB2INSTANCE?} -L"
      rc=$?
      rmrsrc -s "Name like '${DB2INSTANCE?}${NN?}clusterInitiatedStop'" IBM.Test 2> /dev/null
   fi
else
   su - ${DB2INSTANCE} -c "$INSTHOME/sqllib/bin/db2gcf -t 100 -d -p ${NN?} -i ${DB2INSTANCE?} -L"
   rc=$?
   rmrsrc -s "Name like '${DB2INSTANCE?}${NN?}clusterInitiatedStop'" IBM.Test 2> /dev/null
fi

if [[ $rc != 0 ]]; then

   logger -i -p notice -t $PROGNAME "Killing db2 processes"
   kill -9 $(ps -o pid,comm -u $DB2INSTANCE | grep "db2[a-z]" | grep -v db2haicu | awk '{print $1}')
   logger -i -p notice -t $PROGNAME "Removing IPCs"
   su - ${DB2INSTANCE} -c "$INSTHOME/sqllib/bin/db2gcf -t 10 -k -p ${NN?} -i ${DB2INSTANCE?};ipclean -a"
   logger -i -p notice -t $PROGNAME "Completed hard kill ( $DB2INSTANCE, $NN )"
   rc=0
fi

logger -i -p notice -t $PROGNAME "Returning $rc from $0 ( $DB2INSTANCE, $NN )"
return $rc
