#!/bin/ksh -p
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2009
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# VERSION: db2_v97fp4 s110330
# INPUT:  hadrV97_monitor.ksh db2instp db2insts hadrdb [verbose]
#
# OUTPUT: 1 if online, 2 if offline, 0 if not known
#
# NOTES: Can only be used in the context of a TSA environment configured 
#        with the db2 HA Integrated Configuration Utility (db2haicu)
#
#-----------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH
LANG=C
export LANG
logger -i -p notice -t $0 "Entering : $*"

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
DB2HADRINSTANCE1=${1?}
DB2HADRINSTANCE2=${2?}
dbname=${3?}
DB2HADRDBNAME=$(echo ${dbname?} | tr "[a-z]" "[A-Z]")
localhost=$(hostname)
VERBOSE=${4:-noverbose}

#logger -i -p debug -t $0 "PROGNAME: $PROGNAME,PROGPATH: $PROGPATH,DB2HADRINSTANCE1: $DB2HADRINSTANCE1, DB2HADRINSTANCE2: $DB2HADRINSTANCE2, dbname: $dbname, DB2HADRDBNAME: $DB2HADRDBNAME, VERBOSE: $VERBOSE"

if [[ "$VERBOSE" == "verbose" ]]; then
   typeset -ft $(typeset +f)
   set -x
else
   # Close stdout and stderr
   exec 2> /dev/null
   exec 1> /dev/null
   set +x
fi

CT_MANAGEMENT_SCOPE=2
export CT_MANAGEMENT_SCOPE

CLUSTER_APP_LABEL=db2_${DB2HADRINSTANCE1?}_${DB2HADRINSTANCE2?}_${DB2HADRDBNAME?}


###########################################################
# set_candidate_P_instance()
###########################################################
set_candidate_P_instance()
{
   candidate_P_instance=${DB2HADRINSTANCE1?}
   candidate_S_instance=${DB2HADRINSTANCE2?}
   localShorthost=$(hostname | tr "." " " | awk '{print $1}')

# logger -i -p debug -t $0 "candidate_P_instance: $candidate_P_instance, candidate_S_instance: $candidate_S_instance, localShorthost: $localShorthost"

   if [[ ${DB2HADRINSTANCE1?} != ${DB2HADRINSTANCE2?} ]]; then
      rm  -f ${PROGPATH?}/.${CLUSTER_APP_LABEL?}
      if [[ -f ${PROGPATH?}/.${CLUSTER_APP_LABEL?} ]]; then
         candidate_P_instance=$(cat ${PROGPATH?}/.${CLUSTER_APP_LABEL?})
         if [[ ${candidate_P_instance?} == ${DB2HADRINSTANCE1?} ]]; then
            candidate_S_instance=${DB2HADRINSTANCE2?}
         else
            candidate_S_instance=${DB2HADRINSTANCE1?}
         fi
      else
         nn1=$(lsequ -s "Name like '%${DB2HADRINSTANCE1?}_${localShorthost?}%'" | grep "Resource:Node" | awk '{print $3}' | tr ":" " " | tr "{" " " | tr "}" " " | awk '{print $1}')
         if [[ -z "$nn1" ]] ; then
            nn1=$(lsequ -s "Name like '%${DB2HADRINSTANCE1?}_0%'" | grep "Resource:Node" | awk '{print $3}' | tr ":" " " | tr "{" " " | tr "}" " " | awk '{print $1}')
         fi

         nn1s=$(echo $nn1 | tr "." " " | awk '{print $1}')
   
         if [[ ${nn1s?} == ${localShorthost?} ]]; then
            candidate_P_instance=${DB2HADRINSTANCE1?}
            candidate_S_instance=${DB2HADRINSTANCE2?}
         else
            candidate_P_instance=${DB2HADRINSTANCE2?}
            candidate_S_instance=${DB2HADRINSTANCE1?}
         fi
         echo ${candidate_P_instance?} > ${PROGPATH?}/.${CLUSTER_APP_LABEL?}
      fi
   fi

# logger -i -p debug -t $0 "candidate_P_instance: $candidate_P_instance, candidate_S_instance: $candidate_S_instance, localShorthost: $localShorthost, nn1: $nn1, nn1s: $nn1s"

   return 0
}


###########################################################
# monitorhadr()
###########################################################
monitorhadr()
{
   # Get path to home dir
   DB2INSTANCE=${candidate_P_instance?}
   export DB2INSTANCE
   if [[ -z ${INSTHOME} ]] ; then
      userhome=~${DB2INSTANCE?}
      eval userhome=$userhome
      INSTHOME=${userhome}
   fi
   if [[ -z ${INSTHOME} ]] ; then
      INSTHOME=$(cat /etc/passwd | grep "^${DB2INSTANCE?}:" | cut -f 6 -d \:)
   fi

   if [[ -z ${INSTHOME} ]] ; then
      logger -i -p error -t $0 "Can't figure out DB2INSTANCE: $DB2INSTANCE home directory"
      return 0
   fi
   
   manual=-1
   grc=3
   while [ $grc -ne 0 -a $grc -ne 1 ];
   do
      # If IBM.Test resource exists OR reintegration flag file exists, run db2gcf
      usepd=$(lsrsrc -s "Name like 'db2_${DB2HADRDBNAME?}_%${candidate_P_instance?}%'" IBM.Test | grep -v "test_rg" |  grep -c "^resource ")
      flagFileNamePre=db2_${DB2HADRDBNAME?}
      flagFileNamePost1=Reintegrate_${candidate_P_instance?}_${candidate_S_instance?}
      flagFileNamePost2=Reintegrate_${candidate_S_instance?}_${candidate_P_instance?}
      flagFile=$(ls /tmp | grep $flagFileNamePre | grep -c $flagFileNamePost1)
      if [[ $flagFile -eq 0 ]]; then
         flagFile=$(ls /tmp | grep $flagFileNamePre | grep -c $flagFileNamePost2)
      fi 

      # if manual mode, or no state flags, can simply use db2pd to return HADR state ...

      # Use db2pd if in manual mode (manual = 1), OR 
      # if we're in auto mode (or if the mode is unknown), AND there are no IBM.Test resource and flag file
      if [[ $manual -eq 1 || ( ( $manual -eq 0 || $manual -eq -1 ) && ( $usepd -eq 0 && $flagFile -eq 0 ) ) ]]; then
         # for not activated; standby; standard; instance offline - return 2
         hdrRole=$(su - ${candidate_P_instance} -c "db2pd -hadr -db ${DB2HADRDBNAME?}" | egrep "^Primary |^Standby |not activated|not active" | awk '{print $1}')
         logger -i -p debug -t $0 "manual mode, or no state flags, use db2pd: hdrRole:$hdrRole, manual:$manual, usepd:$usepd, flagFile:$flagFile"
         if [[ $hdrRole == "Primary" ]]; then
            grc=0
         elif [[ $hdrRole == '' ]]; then
            # either instance is offline or we're hitting ASLR error
            p_pid=$(ps -u ${candidate_P_instance?} -o args | grep -v grep | grep -c db2sysc)
            if [[ $p_pid -eq 0 ]]; then
               # hadr is also offline on this host
               rc=2
               logger -i -p notice -t $0 "Returning $rc : instance is not available"
               exit $rc 
            fi
            # otherwise
            logger -i -p notice -t $0 "db2pd returns empty due to ASLR, try db2pd again"
         else # database is either Standby, Standard, or not activated 
            grc=1
            
            if [[ $hdrRole == "Standby" ]]; then
               # if standby,peer AND flag then remove flag and resetrsrc 
               # reset Failed Offline resource if reintegration has been completed already
               tmpReint=reint.${candidate_P_instance}.${DB2HADRDBNAME?}.out
               hdrReint=$(ls /tmp | grep -c $tmpReint)
               if [[ $hdrReint -ne 0 ]]; then
                  isPeer=$(su - ${candidate_P_instance} -c "db2pd -hadr -db ${DB2HADRDBNAME?}" | grep -c "Peer ")

                  # If the HADR state is in Peer but TSA resource shows Failed Offline, 
                  # we will reset that resource here
                  if [[ $isPeer == 1 ]]; then
                    rm -f /tmp/${tmpReint?}
                    opState=$(lsrsrc -s "Name = 'db2_${candidate_P_instance?}_${candidate_S_instance?}_${DB2HADRDBNAME?}-rs' AND NodeNameList = '${localhost?}'" IBM.Application OpState | grep OpState | awk '{print $3}')
                    logger -i -p debug -t $0 "lsrsrc for resource 'db2_${candidate_P_instance?}_${candidate_S_instance?}_${DB2HADRDBNAME?}-rs' issued on '${localhost?}', OpState: $opState"
	 
                    # TSA resource state shows FAILED OFFLINE
                    if [[ $opState -eq 3 ]]; then
                        resetrsrc -s "Name = 'db2_${candidate_P_instance?}_${candidate_S_instance?}_${DB2HADRDBNAME?}-rs' AND NodeNameList = '${localhost?}'" IBM.Application
                        logger -i -p notice -t $0 "resetrsrc for resource 'db2_${candidate_P_instance?}_${candidate_S_instance?}_${DB2HADRDBNAME?}-rs' issued on '${localhost?}'" 
                    fi
                    if [[ ${candidate_S_instance?} != ${candidate_P_instance?} ]]; then
                       opState=$(lsrsrc -s "Name = 'db2_${candidate_S_instance?}_${candidate_P_instance?}_${DB2HADRDBNAME?}-rs' AND NodeNameList = '${localhost?}'" IBM.Application OpState | grep OpState | awk '{print $3}')
                       logger -i -p debug -t $0 "lsrsrc for resource 'db2_${candidate_S_instance?}_${candidate_P_instance?}_${DB2HADRDBNAME?}-rs' issued on '${localhost?}', OpState: $opState"
                       if [[ $opState -eq 3 ]]; then
                            resetrsrc -s "Name = 'db2_${candidate_S_instance?}_${candidate_P_instance?}_${DB2HADRDBNAME?}-rs' AND NodeNameList = '${localhost?}'" IBM.Application
                            logger -i -p notice -t $0 "resetrsrc for resource 'db2_${candidate_S_instance?}_${candidate_P_instance?}_${DB2HADRDBNAME?}-rs' issued on '${localhost?}'" 
                       fi  
                    fi  
                  fi
               fi
            fi
         fi
      fi
      
      # If state change during the reading ... re-read
      usepd=$(lsrsrc -s "Name like 'db2_${DB2HADRDBNAME?}_%${candidate_P_instance?}%'" IBM.Test | grep -v "test_rg" |  grep -c "^resource ")
      flagFile=$(ls /tmp | grep $flagFileNamePre | grep -c $flagFileNamePost1)
      if [[ $flagFile -eq 0 ]]; then
         flagFile=$(ls /tmp | grep $flagFileNamePre | grep -c $flagFileNamePost2)
      fi 
      if [[ $usepd -ne 0 ]] ; then
         logger -i -p debug -t $0 "Must try reading with db2gcf"
      fi

      # If not in manual mode (or mode is unknown), AND
      # there are IBM.Test resource or flag file, OR db2pd was unsuccessful (grc=3),
      # use db2gcf
      if [[ ( $manual -eq 0 || $manual -eq -1 ) && ( $usepd -ne 0 || $flagFile -ne 0 || $grc -eq 3 )]]; then
         manual=$(lssamctrl | grep -c "Manual")
         if [[ $manual == 0 ]]; then
            logger -i -p debug -t $0 "auto mode, and state flags or db2pd fails, use db2gcf: manual:$manual, usepd:$usepd, flagFile:$flagFile, grc:$grc"
            # Want db2gcf to timeout prior to the monitor method timeout ... 
            # gcfTimeOut=15
            # Want db2gcf to timeout prior to the monitor method timeout ... 
            mcto=29
            mcto=$(lsrsrc -s "Name = 'db2_${candidate_P_instance?}_${candidate_S_instance?}_${DB2HADRDBNAME?}-rs' AND NodeNameList = '${localhost?}'" IBM.Application  MonitorCommandTimeout | grep MonitorCommandTimeout | awk '{print $3}')
            if [[ $mcto == "" ]]; then
               #try the other way
               mcto=$(lsrsrc -s "Name = 'db2_${candidate_S_instance?}_${candidate_P_instance?}_${DB2HADRDBNAME?}-rs' AND NodeNameList = '${localhost?}'" IBM.Application  MonitorCommandTimeout | grep MonitorCommandTimeout | awk '{print $3}')
            fi

            if [[ $mcto -lt 29 ]]; then
               mcto=29
            fi
            let gcfTimeOut=mcto/2
            su - ${candidate_P_instance?} -c "$INSTHOME/sqllib/bin/db2gcf -t ${gcfTimeOut?} -s -i ${candidate_P_instance?} -i ${candidate_S_instance?} -h ${DB2HADRDBNAME?}"
            grc=$?
            logger -i -p debug -t $0 "su - ${candidate_P_instance?} -c $INSTHOME/sqllib/bin/db2gcf -t ${gcfTimeOut?} -s -i ${candidate_P_instance?} -i ${candidate_S_instance?} -h ${DB2HADRDBNAME?} returns $grc"
         else
            logger -i -p debug -t $0 "Domain is placed in manual mode, skip db2gcf call."
         fi
      fi
   done


   if [[ $grc == 0 ]]; then
      # Only good db2gcf result maps to TSA ONLINE
      rc=1
   elif [[ $grc == 1 ]]; then
      # OFFLINE
      rc=2
   fi 
   
   return $rc
}

#######################################################
# main()
#######################################################
main()
{
   # What instance is at this node?
   set_candidate_P_instance
   
   # We need to checking if the instance start is complete here. To minimize
   # the dependency on the checking for the db2 start script, we first check
   # if there is a db2sysc process. If not, no need to go further. If there is
   # a db2sysc process, check if the script is still running. 
   p_pid=$(ps -u ${candidate_P_instance?} -o args | grep -c "^db2sysc")
   if [ $p_pid -eq 0 ]; then
      logger -i -p info -t $PROGNAME "$0 $DB2INSTANCE is offline, hadr monitor must return offline"
      rc=2
      return $rc
   elif [ $p_pid -ne 0 ]; then
      isInstanceStartRunning=$(ps -ef | grep db2V97_start.ksh | grep ${candidate_P_instance?} | wc -l)
      if [[ $isInstanceStartRunning -ne 0 ]]; then
         logger -i -p notice -t $0 "db2sysc started but Instance start still running...return OFFLINE for HADR database ${DB2HADRDBNAME?}"
         rc=2
         return $rc 
      fi
   fi
  
   # Should re-integration be performed?
   nn=$(lsrsrc -s "Name = 'db2_${DB2HADRDBNAME?}_${localShorthost?}_Reintegrate_${candidate_P_instance?}_${candidate_S_instance?}'" IBM.Test Name | grep -c Name)
   if [[ $nn -eq 0 ]]; then
      if [[ ${candidate_S_instance?} != ${candidate_P_instance} ]]; then
         # Unfortunately, must check both permutations of the naming scheme ...
         nn=$(lsrsrc -s "Name = 'db2_${DB2HADRDBNAME?}_${localShorthost?}_Reintegrate_${candidate_S_instance?}_${candidate_P_instance?}'" IBM.Test Name | grep -c Name)
      fi
   fi
   # If so, all apps must be forced...
   if [[ $nn -ne 0 ]]; then
      # create reset failed offline
      tmpReint=reint.${candidate_P_instance?}.${DB2HADRDBNAME?}.out
      su - ${candidate_P_instance?} -c "/bin/ksh /usr/sbin/rsct/sapolicies/db2/forceAllApps $candidate_P_instance $DB2HADRDBNAME"
      touch /tmp/${tmpReint}
      logger -i -p debug -t $0 "Reintegration memory file created under /tmp for db ${DB2HADRDBNAME?}"
   fi
   monitorhadr
   return $rc
}

if [[ "$VERBOSE" == "verbose" ]]; then
   typeset -ft $(typeset +f)
fi

#######################################################
main "${@:-}"
logger -i -p notice -t $0 "Returning $rc : $*"

exit $rc
