-- -*- sql -*-
--
-- Sample DDL to create audit tables for Version 9.5
--
--   -> assumes db2start issued
--   -> assumes connection to a database exists
--   -> assumes an 8K tablespace is available for use
--   -> assumes called by "db2 -tf db2audit.ddl"
--

--
-- To remind users how to use this file!
--
ECHO                              ;
ECHO ******* IMPORTANT ********** ;
ECHO                              ;
ECHO USAGE: db2 -tf db2audit.ddl  ;
ECHO                              ;
ECHO ******* IMPORTANT ********** ;
ECHO                              ;
ECHO                              ;
--
--

--
-- AUDIT CATEGORY
--

CREATE TABLE AUDIT ( TIMESTAMP CHAR(26),
                     CATEGORY CHAR(8),
                     EVENT VARCHAR(32),
                     CORRELATOR INTEGER,
                     STATUS INTEGER,
                     USERID VARCHAR(1024),
                     AUTHID VARCHAR(128),
                     DATABASE CHAR(8),
                     NODENUM SMALLINT,
                     COORDNUM SMALLINT,
                     APPID VARCHAR(255),
                     APPNAME VARCHAR(1024),
                     PKGSCHEMA VARCHAR(128),
                     PKGNAME VARCHAR(128),
                     PKGSECNUM SMALLINT,
                     PKGVER VARCHAR(64),
                     LCLTRANSID VARCHAR(10) FOR BIT DATA,
                     GLBLTRANSID VARCHAR(30) FOR BIT DATA,
                     CLNTUSERID VARCHAR(255),
                     CLNTWRKSTNAME VARCHAR(255),
                     CLNTAPPNAME VARCHAR(255),
                     CLNTACCSTRING VARCHAR(255),
                     TRSTCTXNAME VARCHAR(255),
                     CONTRSTTYPE INTEGER,
                     ROLEINHERITED VARCHAR(128),
                     POLNAME VARCHAR(128),
                     POLASSOCOBJTYPE CHAR(1),
                     POLASSOCSUBOBJTYPE CHAR(1),
                     POLASSOCNAME VARCHAR(128),
                     POLASSOCOBJSCHEMA VARCHAR(128),
                     AUDITSTATUS CHAR(1),
                     CHECKINGSTATUS CHAR(1),
                     CONTEXTSTATUS CHAR(1),
                     EXECUTESTATUS CHAR(1),
                     EXECUTEDATA CHAR(1),
                     OBJMAINTSTATUS CHAR(1),
                     SECMAINTSTATUS CHAR(1),
                     SYSADMINSTATUS CHAR(1),
                     VALIDATESTATUS CHAR(1),
                     ERRORTYPE CHAR(8),
                     DATAPATH VARCHAR(1024),
                     ARCHIVEPATH VARCHAR(1024)); 

--
-- CHECKING CATEGORY
--

CREATE TABLE CHECKING ( TIMESTAMP CHAR(26),
                        CATEGORY CHAR(8),
                        EVENT VARCHAR(32),
                        CORRELATOR INTEGER,
                        STATUS INTEGER,
                        DATABASE CHAR(8),
                        USERID VARCHAR(1024),
                        AUTHID VARCHAR(128),
                        NODENUM SMALLINT,
                        COORDNUM SMALLINT,
                        APPID VARCHAR(255),
                        APPNAME VARCHAR(1024),
                        PKGSCHEMA VARCHAR(128),
                        PKGNAME VARCHAR(128),
                        PKGSECNUM SMALLINT,
                        OBJSCHEMA VARCHAR(128),
                        OBJNAME VARCHAR(128),
                        OBJTYPE VARCHAR(32),
                        ACCESSAPP CHAR(34),
                        ACCESSATT CHAR(34),
                        PKGVER VARCHAR(64),
                        CHKAUTHID VARCHAR(128),
                        LCLTRANSID VARCHAR(10) FOR BIT DATA,
                        GLBLTRANSID VARCHAR(30) FOR BIT DATA,
                        CLNTUSERID VARCHAR(255),
                        CLNTWRKSTNAME VARCHAR(255),
                        CLNTAPPNAME VARCHAR(255),
                        CLNTACCSTRING VARCHAR(255),
                        TRSTCTXNAME VARCHAR(255),
                        CONTRSTTYPE INTEGER,
                        ROLEINHERITED VARCHAR(128));


--
-- OBJMAINT CATEGORY
--

CREATE TABLE OBJMAINT ( TIMESTAMP CHAR(26),
                        CATEGORY CHAR(8),
                        EVENT VARCHAR(32),
                        CORRELATOR INTEGER,
                        STATUS INTEGER,
                        DATABASE CHAR(8),
                        USERID VARCHAR(1024),
                        AUTHID VARCHAR(128),
                        NODENUM SMALLINT,
                        COORDNUM SMALLINT,
                        APPID VARCHAR(255),
                        APPNAME VARCHAR(1024),
                        PKGSCHEMA VARCHAR(128),
                        PKGNAME VARCHAR(128),
                        PKGSECNUM SMALLINT,
                        OBJSCHEMA VARCHAR(128),
                        OBJNAME VARCHAR(128),
                        OBJTYPE VARCHAR(32),
                        PACKVER VARCHAR(64),
                        SECPOLNAME VARCHAR(128),
                        ALTERACTION VARCHAR(32),
                        PROTCOLNAME VARCHAR(128),
                        COLSECLABEL VARCHAR(128),
                        SECCOLNAME VARCHAR(128),
                        LCLTRANSID VARCHAR(10) FOR BIT DATA,
                        GLBLTRANSID VARCHAR(30) FOR BIT DATA,
                        CLNTUSERID VARCHAR(255),
                        CLNTWRKSTNAME VARCHAR(255),
                        CLNTAPPNAME VARCHAR(255),
                        CLNTACCSTRING VARCHAR(255),
                        TRSTCTXNAME VARCHAR(255),
                        CONTRSTTYPE INTEGER,
                        ROLEINHERITED VARCHAR(128),
                        MODULENAME VARCHAR(128));


--
-- SECMAINT CATEGORY
--

CREATE TABLE SECMAINT ( TIMESTAMP CHAR(26),
                        CATEGORY CHAR(8),
                        EVENT VARCHAR(32),
                        CORRELATOR INTEGER,
                        STATUS INTEGER,
                        DATABASE CHAR(8),
                        USERID VARCHAR(1024),
                        AUTHID VARCHAR(128),
                        NODENUM SMALLINT,
                        COORDNUM SMALLINT,
                        APPID VARCHAR(255),
                        APPNAME VARCHAR(1024),
                        PKGSCHEMA VARCHAR(128),
                        PKGNAME VARCHAR(128),
                        PKGSECNUM SMALLINT,
                        OBJSCHEMA VARCHAR(128),
                        OBJNAME VARCHAR(128),
                        OBJTYPE VARCHAR(32),
                        GRANTOR VARCHAR(128),
                        GRANTEE VARCHAR(128),
                        GRANTEETYPE VARCHAR(32),
                        PRIVAUTH CHAR(34),
                        PKGVER VARCHAR(64),
                        ACCESSTYPE VARCHAR(32),
                        ASSUMEAUTHID VARCHAR(128),
                        LCLTRANSID VARCHAR(10) FOR BIT DATA,
                        GLBLTRANSID VARCHAR(30) FOR BIT DATA,
                        GRANTORTYPE VARCHAR(32),
                        CLNTUSERID VARCHAR(255),
                        CLNTWRKSTNAME VARCHAR(255),
                        CLNTAPPNAME VARCHAR(255),
                        CLNTACCSTRING VARCHAR(255),
                        TRSTCTXUSER VARCHAR(128),
                        TRSTCTXUSERAUTH INTEGER,
                        TRSTCTXNAME VARCHAR(255),
                        CONTRSTTYPE INTEGER,
                        ROLEINHERITED VARCHAR(128));

--
-- SYSADMIN CATEGORY
--

CREATE TABLE SYSADMIN ( TIMESTAMP CHAR(26),
                        CATEGORY CHAR(8),
                        EVENT VARCHAR(32),
                        CORRELATOR INTEGER,
                        STATUS INTEGER,
                        DATABASE CHAR(8),
                        USERID VARCHAR(1024),
                        AUTHID VARCHAR(128),
                        NODENUM SMALLINT,
                        COORDNUM SMALLINT,
                        APPID VARCHAR(255),
                        APPNAME VARCHAR(1024),
                        PKGSCHEMA VARCHAR(128),
                        PKGNAME VARCHAR(128),
                        PKGSECNUM SMALLINT,
                        PKGVER VARCHAR(64),
                        LCLTRANSID VARCHAR(10) FOR BIT DATA,
                        GLBLTRANSID VARCHAR(30) FOR BIT DATA,
                        CLNTUSERID VARCHAR(255),
                        CLNTWRKSTNAME VARCHAR(255),
                        CLNTAPPNAME VARCHAR(255),
                        CLNTACCSTRING VARCHAR(255),
                        TRSTCTXNAME VARCHAR(255),
                        CONTRSTTYPE INTEGER,
                        ROLEINHERITED VARCHAR(128));
                          
--
-- VALIDATE CATEGORY
--

CREATE TABLE VALIDATE ( TIMESTAMP CHAR(26),
                        CATEGORY CHAR(8),
                        EVENT VARCHAR(32),
                        CORRELATOR INTEGER,
                        STATUS INTEGER,
                        DATABASE CHAR(8),
                        USERID VARCHAR(1024),
                        AUTHID VARCHAR(128),
                        EXECID VARCHAR(1024),
                        NODENUM SMALLINT,
                        COORDNUM SMALLINT,
                        APPID VARCHAR(255),
                        APPNAME VARCHAR(1024),
                        AUTHTYPE VARCHAR(32),
                        PKGSCHEMA VARCHAR(128),
                        PKGNAME VARCHAR(128),
                        PKGSECNUM SMALLINT,
                        PKGVER VARCHAR(64),
                        PLUGINNAME VARCHAR(32),
                        LCLTRANSID VARCHAR(10) FOR BIT DATA,
                        GLBLTRANSID VARCHAR(30) FOR BIT DATA,
                        CLNTUSERID VARCHAR(255),
                        CLNTWRKSTNAME VARCHAR(255),
                        CLNTAPPNAME VARCHAR(255),
                        CLNTACCSTRING VARCHAR(255),
                        TRSTCTXNAME VARCHAR(255),
                        CONTRSTTYPE INTEGER,
                        ROLEINHERITED VARCHAR(128));

--
-- CONTEXT CATEGORY
--

CREATE TABLE CONTEXT ( TIMESTAMP CHAR(26),
                       CATEGORY CHAR(8),
                       EVENT VARCHAR(32),
                       CORRELATOR INTEGER,
                       DATABASE CHAR(8),
                       USERID VARCHAR(1024),
                       AUTHID VARCHAR(128),
                       NODENUM SMALLINT,
                       COORDNUM SMALLINT,
                       APPID VARCHAR(255),
                       APPNAME VARCHAR(1024),
                       PKGSCHEMA VARCHAR(128),
                       PKGNAME VARCHAR(128),
                       PKGSECNUM SMALLINT,
                       STMTTEXT CLOB(8M),
                       PKGVER VARCHAR(64),
                       LCLTRANSID VARCHAR(10) FOR BIT DATA,
                       GLBLTRANSID VARCHAR(30) FOR BIT DATA,
                       CLNTUSERID VARCHAR(255),
                       CLNTWRKSTNAME VARCHAR(255),
                       CLNTAPPNAME VARCHAR(255),
                       CLNTACCSTRING VARCHAR(255),
                       TRSTCTXNAME VARCHAR(255),
                       CONTRSTTYPE INTEGER,
                       ROLEINHERITED VARCHAR(128));


--
-- EXECUTE CATEGORY
--

CREATE TABLE EXECUTE ( TIMESTAMP CHAR(26),
                       CATEGORY CHAR(8),
                       EVENT VARCHAR(32),
                       CORRELATOR INTEGER,
                       STATUS INTEGER,
                       DATABASE CHAR(8),
                       USERID VARCHAR(1024),
                       AUTHID VARCHAR(128),
                       SESSNAUTHID VARCHAR(128),
                       NODENUM SMALLINT,
                       COORDNUM SMALLINT,
                       APPID VARCHAR(255),
                       APPNAME VARCHAR(1024),
                       CLNTUSERID VARCHAR(255),
                       CLNTWRKSTNAME VARCHAR(255),
                       CLNTAPPNAME VARCHAR(255),
                       CLNTACCSTRING VARCHAR(255),
                       TRSTCTXNAME VARCHAR(255),
                       CONTRSTTYPE INTEGER,
                       ROLEINHERITED VARCHAR(128),
                       PKGSCHEMA VARCHAR(128),
                       PKGNAME VARCHAR(128),
                       PKGSECNUM SMALLINT,
                       PKGVER VARCHAR(64),
                       LCLTRANSID VARCHAR(10) FOR BIT DATA,
                       GLBLTRANSID VARCHAR(30) FOR BIT DATA,
                       UOWID BIGINT,
                       ACTIVITYID BIGINT,
                       STMTINVOCID BIGINT,
                       STMTNESTLVL BIGINT,
                       ACTIVITYTYPE VARCHAR(32),
                       STMTTEXT CLOB(8M),
                       STMTISOLATIONLVL CHAR(8),
                       COMPENVDESC BLOB(8K),
                       ROWSMODIFIED INTEGER,
                       ROWSRETURNED BIGINT,
                       SAVEPOINTID BIGINT,
                       STMTVALINDEX INTEGER,
                       STMTVALTYPE CHAR(16),
                       STMTVALDATA CLOB(128K),
                       STMTVALEXTENDEDINDICATOR INTEGER,
                       LOCAL_START_TIME CHAR(26));
